#ifndef ERTOOL_TYPES_H
#define ERTOOL_TYPES_H

#include <vector>
#include <map>
#include <string>
#include <cstddef>
#include <string>
namespace ertool {

  typedef size_t RecoID_t;     ///< A unique reconstructed object (input) identifier variable type
  typedef size_t FlashID_t;    ///< A unique flash object (input) identifier variable type
  typedef size_t NodeID_t;     ///< A unique particle identifier variable type
  typedef int    Generation_t; ///< A layer number in particle graph (0 = Primary, kDefaultLayer = not yet assessed )

  /// Defines an input shower/track original product information
  typedef std::pair<size_t, std::string> RecoInputID_t;

  /// Defines a process information container
  typedef std::map<std::string, std::string> ProcInfo_t;

  /// Defines a type for a combinatory index expression
  typedef std::vector<NodeID_t> Combination_t;

  /// Defines a set of combinations
  typedef std::vector<Combination_t> CombinationSet_t;

  /// Reconstructed object type associated with each particle
  enum RecoType_t {
    kInvisible, ///< No reconstructed information
    kShower,    ///< Associated with a shower object
    kTrack,     ///< Associated with a track object
    kINVALID_RECO_TYPE
  };

  /// Origin type, directly copied from mcshower + mctrack + mctruth Origin_t
  /// This is truth whether the origin is Genie (neutrino) or CRY/Corsika (cosmic)
  enum Origin_t {
    kUnknownOrigin,           ///< ???
    kBeamNeutrino,      ///< Beam neutrinos
    kCosmicRay,         ///< Cosmic rays
    kSuperNovaNeutrino, ///< Supernova neutrinos
    kSingleParticle     ///< single particles thrown at the detector
  };

  /// Relation Type
  enum RelationType_t {
    kChild,
    kParent,
    kSibling,
    kUnrelated,
    kINVALID_RELATION_TYPE
  };

  /// "Process Type" that created this particle
  enum ProcessType_t {
    kUnknown,
    /// Particles coming out of the neutrino interaction
    kNuCCQE,         //1001
    kNuNC,           //1002
    kNuCC1Pi,        //1003,5,10,12
    kNuCC1PiZero,    //1004,11
    kNuNC1Pi,        //1007,9,14,16
    kNuNC1PiZero,    //1008,15
    kNuCCDeltaRes,   //1017-21,28-32
    kNuNCDeltaRes,   //1022-27,33-38
    kNuDIS,          //1091-1092
    kNuOther,        //every other types of neutrino interactions
    /// Other particles
    kMichel,         //michel electrons "muMinusCaptureAtRest"
    kMuIoni,         //particles (electrons?) from muon ionization "muIoni"
    kCosmic,         //Cosmics
    kCosmicOrphan,   //Cosmic specifically tagged by an orphan-finding algo
    kPiZeroDecay,    //daughters of pizero decays (gammas usually)
    //Neutrino Production Processes
    kK0L,//Processes follow
    // K0L -> nue pi- e+,
    // K0L -> nuebar pi+ e-
    // K0L -> numu pi- mu+
    // K0L -> numubar pi+ mu-
    kKCharged,//Processes follow
    // K+  -> numu mu+
    // K+  -> nue pi0 e+
    // K+  -> numu pi0 mu+
    // K-  -> numubar mu-
    // K-  -> nuebar pi0 e-
    // K-  -> numubar pi0 mu-
    kMuDecay,//Processes follow
    // mu+ -> numubar nue e+
    // mu- -> numu nuebar e-
    kPionDecay,//Processes follow
    // pi+ -> numu mu+
    // pi- -> numubar mu-
    kPiZeroMID,     //hacky addition for downstream algos that determine a nue is actually a pi0 MID
    kINVALID_PROCESS_TYPE
  };

  namespace io {
    enum Mode_t {
      kUNDEFINED, ///< undefined mode
      kREAD,  ///< read mode
      kWRITE, ///< write mode
      kBOTH   ///< both read & write mode
    };

    enum StreamType_t {
      kEmptyStream,
      kROOT,
      kROOTTuple
    };
  }

  namespace msg {
    enum Level_t {
      kDEBUG = 0,    ///< Message level ... useful to debug a crash
      kINFO,         ///< Debug info but not the lowest level
      kNORMAL,       ///< Normal stdout
      kWARNING,      ///< notify a user in the standard operation mode for an important finding.
      kERROR,        ///< notify a user when something is clearly wrong
      kEXCEPTION,    ///< Exception!
      kMSG_TYPE_MAX
    };
  }
}

#endif
